// Fichier présentant la structure de classes gérant une ressource
// Il s'agit bien sûr d'un exemple non-fonctionnel, avec des noms et des fonctions inventées.

/************************** UneRessource.hpp **************************/

class UneRessource {
private:
	int identifiant;
	char* buffer;

protected:
	// Méthode privée, qui n'a pas à être utilisée en dehors de la classe
	bool envoyer_commande (std::string commande);

public:
	// Se connecte à la ressource à partir de son nom.
	UneRessource (const char* adresse_ressource);
	// Constructeur par copie, et copie par assignation
	UneRessource (const UneRessource&);
	UneRessource& operator= (const UneRessource&);
	// Destructeur : se déconnecte de la ressource
	~UneRessource ();

	// Méthode et sa surcharge
	void faire_un_truc ();
	void faire_un_truc (int truc);

	double mesurer_un_truc ();
	
	// Donne le statut; n'evoie pas de commande à la ressource, ie. ne modifie rien => méthode constante
	bool statut () const;
}

/************************** UneRessource.cpp **************************/

UneRessource::UneRessource (const char* adresse_ressource) {
	// se connecte à la ressource
	identifiant = api_connect(adresse_ressource);
	// succès ? si non, lance une exception; le buffer ne sera pas alloué, ne créant pas de fuite mémoire
	if (identifiant < 0):
		throw std::runtime_error("catastrophe !");

	buffer = new char[100];
}

UneRessource::~UneRessource () {
	api_disconnect(identifiant);
	delete[] buffer;
}

double UneRessource::mesurer_un_truc () {
	do {
		attendre();
	} while (this->statut() == false);
	this->envoyer_commande("MEASURE TRUC");
	int r = api_recieve(identifiant, buffer, 100);
	if (identifiant < 0):
		throw std::runtime_error("catastrophe !");
	// [ décode ce qu'il y a dans le buffer ]
	return 42;
}

UneRessource::UneRessource (const UneRessource& o) : identifiant(o.identifiant) {
	// il y a juste à copier l'identifiant ressource et à dire à l'API qu'il y a un nouveau client
	api_refcount_increase(identifiant);
	buffer = new char[100];
}

UneRessource& operator= (const UneRessource& o) {
	if (this != &o) {
		api_disconnect(identifiant);
		identifiant = o.identifiant;
		api_refcount_increase(identifiant);
	}
}
